#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#define N 26

int main (int argc, char **argv)
{
	int size, rank;
	int street[N+2], newstreet[N+2];
	int i,j,left,right, istart, iend;
	float x, p, v;
	int scounts[4], displs[4];
	MPI_Comm MyComm;
	int ndims, dims[1], periods[1];
	int srank,drank,nrank;
	
	MPI_Init(&argc, &argv);
	
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	
	ndims = 1;
	dims[0] = size;
	periods[0] = 1;
	MPI_Cart_create(MPI_COMM_WORLD,1,dims,periods,1,&MyComm);
	MPI_Comm_rank(MyComm, &nrank);
	
	istart = N * rank / size + 1;
	iend = N * (rank + 1) / size;
	
	MPI_Cart_shift(MyComm, 0, -1, &left, &right);
	printf("left: %d rank: %d right: %d\n",left,nrank,right);
	
	p = 0.5;   
	if (rank == 0) {
		for (i = 1; i <= N; i++) {
			x = random() / ((float)RAND_MAX+1);
			if ( x < p)
				street[i] = 1;
			else
				street[i] = 0;
		}      
   

		for (i=0;i <= size - 1; i++) {
			scounts[i] = N * (i+1) / size - N*i/size;
			displs[i] = N * i / size + 1;
		}
	}

   // Bcast
   MPI_Bcast(street,N+2,MPI_INT,
             0,MPI_COMM_WORLD);
for(j = 1; j <= 5; j++) {

	// Copy boundary conditions
	MPI_Sendrecv(&street[iend],1,MPI_INT,right,0,
                &street[istart-1],1,MPI_INT,left,0,
                MyComm, MPI_STATUS_IGNORE);

	MPI_Sendrecv(&street[istart],1,MPI_INT,left,0,
                &street[iend+1],1,MPI_INT,right,0,
		MyComm, MPI_STATUS_IGNORE);
   
	// Apply cellural automata
	for ( i = istart; i <= iend; i++)
		if (street[i] == 0)
			newstreet[i] = street[i-1];
		else
			newstreet[i] = street[i+1];

	v = 0.0;
	for (i = istart; i <= iend; i++)
		if (street[i] != newstreet[i])
			v = v + 1;

	for (i = istart; i <= iend; i++)
		street[i] = newstreet[i];

	MPI_Gatherv(&street[istart],iend-istart+1,MPI_INT,newstreet,scounts,
		displs,MPI_INT,0,MyComm);

	if (rank == 0) {     
		for (i=1;i<=N;i++)
			printf("%d",newstreet[i]);
		printf("\n");
	} 
}

	MPI_Finalize();
   
	return 0;
}
